# -*- coding: utf-8 -*-
"""
    korean.__main__
    ~~~~~~~~~~~~~~~

    Command-line tools.

    :copyright: (c) 2012 by Heungsub Lee
    :license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import
import contextlib
import sys

from baker import Baker

from . import l10n


baker = Baker()


@contextlib.contextmanager
def file_or_stdin(path):
    f = open(path) if path is not None else sys.stdin
    yield f
    f.close()


@baker.command
def proofread(path=None, charset='utf-8'):
    with file_or_stdin(path) as f:
        for line in f.xreadlines():
            print l10n.proofread(line.decode(charset)),


@baker.command
def validate(path=None, charset='utf-8'):
    pass


if __name__ == '__main__':
    baker.run()



# -*- coding: utf-8 -*-
"""
    korean.hangul
    ~~~~~~~~~~~~~

    Processing a string written by Hangul. All code of here is based on
    `hangul.py
    <https://raw.github.com/sublee/hangulize/master/hangulize/hangul.py>`_ by
    `Hye-Shik Chang <http://openlook.org/>`_ at 2003.

    :copyright: (c) 2012 by Heungsub Lee and 2003 by Hye-Shik Chang
    :license: BSD, see LICENSE for more details.
"""


__all__ = ['char_offset', 'is_hangul', 'is_vowel', 'is_consonant',
           'is_initial', 'is_final', 'get_initial', 'get_vowel', 'get_final',
           'split_char', 'join_char']


def S(*sequences):
    def to_tuple(sequence):
        if not sequence:
            return (sequence,)
        return tuple(sequence)
    return sum(map(to_tuple, sequences), ())
VOWELS = S(u'ㅏㅐㅑㅒㅓㅔㅕㅖㅗㅘㅙㅚㅛㅜㅝㅞㅟㅠㅡㅢㅣ')
CONSONANTS = S(u'ㄱㄲㄳㄴㄵㄶㄷㄸㄹㄺㄻㄼㄽㄾㄿㅀㅁㅂㅃㅄㅅㅆㅇㅈㅉㅊㅋㅌㅍㅎ')
INITIALS = S(u'ㄱㄲㄴㄷㄸㄹㅁㅂㅃㅅㅆㅇㅈㅉㅊㅋㅌㅍㅎ')
FINALS = S(u'', u'ㄱㄲㄳㄴㄵㄶㄷㄹㄺㄻㄼㄽㄾㄿㅀㅁㅂㅄㅅㅆㅇㅈㅊㅋㅌㅍㅎ')
LETTER_ELEMENTS = (INITIALS, VOWELS, FINALS)
HANGUL_RANGE = xrange(ord(u'가'), ord(u'힣') + 1)
FIRST_HANGUL = HANGUL_RANGE[0]
del S


def char_offset(char):
    """Returns Hangul character offset from "가"."""
    if isinstance(char, int):
        offset = char
    else:
        assert len(char) == 1
        assert is_hangul(char)
        offset = ord(char) - FIRST_HANGUL
    assert offset < len(HANGUL_RANGE)
    return offset


def is_hangul(char):
    """Checks if the given character is written in Hangul."""
    return ord(char) in HANGUL_RANGE


def is_vowel(char):
    """Checks if the given character is a vowel of Hangul."""
    return char in VOWELS


def is_consonant(char):
    """Checks if the given character is a consonant of Hangul."""
    return char in CONSONANTS


def is_initial(char):
    """Checks if the given character is an initial consonant of Hangul."""
    return char in INITIALS


def is_final(char):
    """Checks if the given character is a final consonant of Hangul. The final
    consonants contain what a joined multiple consonant and empty character.
    """
    return char in FINALS


def get_initial(char):
    """Returns an initial consonant from the given character."""
    if is_initial(char):
        return char
    return INITIALS[int(char_offset(char) / (len(VOWELS) * len(FINALS)))]


def get_vowel(char):
    """Returns a vowel from the given character."""
    if is_vowel(char):
        return char
    return VOWELS[int(char_offset(char) / len(FINALS)) % len(VOWELS)]


def get_final(char):
    """Returns a final consonant from the given character."""
    if is_final(char):
        return char
    return FINALS[char_offset(char) % len(FINALS)]


def split_char(char):
    """Splits the given character to a tuple where the first item is the
    initial consonant and the second the vowel and the third the final.
    """
    code = char_offset(char)
    return (get_initial(code), get_vowel(code), get_final(code))


def join_char(splitted):
    """Joins a tuple in the form ``(initial, vowel, final)`` to a Hangul
    character.
    """
    assert len(splitted) == len(LETTER_ELEMENTS)
    if not (splitted[0] and splitted[1]):
        return splitted[0] or splitted[1]
    indexes = [tuple.index(*args) for args in zip(LETTER_ELEMENTS, splitted)]
    offset = (indexes[0] * len(VOWELS) + indexes[1]) * len(FINALS) + indexes[2]
    return unichr(FIRST_HANGUL + offset)



# -*- coding: utf-8 -*-
"""
    korean
    ~~~~~~

    A library for Korean morphology.

    :copyright: (c) 2012 by Heungsub Lee
    :license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import

from . import hangul, l10n, morphology
from .morphology import Morpheme, Noun, NumberWord, Loanword, Particle, \
                        Substantive


__copyright__ = 'Copyright 2012 by Heungsub Lee'
__version__ = '0.1.4'
__license__ = 'BSD'
__author__ = 'Heungsub Lee'
__author_email__ = 'h''@''subl.ee'
__url__ = 'http://packages.python.org/korean'
__all__ = ['hangul', 'l10n', 'morphology', 'Morpheme', 'Noun', 'NumberWord',
           'Loanword', 'Particle', 'Substantive']


def load_data():
    import json
    import os
    with open(os.path.join(os.path.dirname(__file__), 'data.json')) as f:
        data = json.load(f)
    # register allomorphic particles
    for forms in data['allomorphic_particles'].itervalues():
        particle = Particle(*forms)
        for form in forms:
            Particle.register(form, particle)
    # register numbers and digits
    for number, form in data['numbers'].iteritems():
        NumberWord.__numbers__[int(number)] = form
    for digit, form in data['digits'].iteritems():
        NumberWord.__digits__[int(digit)] = form


load_data()



# -*- coding: utf-8 -*-
"""
    korean.morphology.morpheme
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2012 by Heungsub Lee
    :license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import

from ..hangul import get_final, is_hangul


__all__ = ['Morpheme']


class MorphemeMetaclass(type):

    def __new__(meta, name, bases, attrs):
        from . import Morphology
        cls = type.__new__(meta, name, bases, attrs)
        cls._registry = {}
        Morphology._register_morpheme(cls)
        return cls

    def __call__(cls, *forms):
        if len(forms) == 1:
            try:
                return cls.get(forms[0])
            except KeyError:
                pass
        return super(MorphemeMetaclass, cls).__call__(*forms)


class Morpheme(object):
    """This class presents a morpheme (형태소) or allomorph (이형태). It
    can have one or more forms. The first form means the basic allomorph
    (기본형).

    :param forms: each forms of allomorph. the first form will be basic
                  allomorph.
    """

    __metaclass__ = MorphemeMetaclass

    _registry = None

    def __init__(self, *forms):
        assert all([isinstance(form, unicode) for form in forms])
        self.forms = forms

    @classmethod
    def get(cls, key):
        """Returns a pre-defined morpheme object by the given key."""
        return cls._registry[key]

    @classmethod
    def register(cls, key, obj):
        """Registers a pre-defined morpheme object to the given key."""
        cls._registry[key] = obj

    def read(self):
        """Every morpheme class would implement this method. They should make a
        morpheme to the valid Korean text with Hangul.
        """
        return unicode(self)

    def basic(self):
        """The basic form of allomorph."""
        return self.forms[0]

    def __unicode__(self):
        return self.basic()

    def __str__(self):
        return unicode(self).encode('utf-8')

    def __getitem__(self, i):
        return unicode(self)[i]

    def __getslice__(self, start, stop, step=None):
        return unicode(self)[start:stop:step]

    def __format__(self, suffix):
        return u'{0!s}{1}'.format(self, suffix)

    def __repr__(self):
        return '{0}({1!s})'.format(type(self).__name__, str(self))



class Registry(object):

    @classmethod
    def get(cls, key):
        try:
            return cls._registry[key]
        except (AttributeError, KeyError):
            raise LookupError('Not found %r in %s' % (key, cls.__name__))

    @classmethod
    def register(cls, key, obj):
        try:
            cls._registry[key] = obj
        except AttributeError:
            cls._registry = {key: obj}



# -*- coding: utf-8 -*-
"""
    korean.morphology.particle
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2012 by Heungsub Lee
    :license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import

from . import define_allomorph_picker
from .morpheme import Morpheme
from .substantive import Noun, NumberWord, Loanword
from .. import hangul


__all__ = ['Particle']


class Particle(Morpheme):
    """Particle (조사) is a postposition in Korean. Some particles are
    allomorph such as 을/를, 이/가. These forms follow forward syllable ends
    what phoneme; a vowel, a consonant, or a Rieul (ㄹ).
    """

    def __init__(self, after_vowel, after_consonant=None, after_rieul=None):
        if after_rieul:
            forms = (after_vowel, after_consonant, after_rieul)
        elif after_consonant:
            forms = (after_vowel, after_consonant)
        else:
            forms = (after_vowel,)
        super(Particle, self).__init__(*forms)

    @classmethod
    def get(cls, key):
        try:
            return super(Particle, cls).get(key)
        except KeyError:
            return cls.guess(key)

    @classmethod
    def guess(cls, key):
        def compare(x, y):
            return -cmp(len(x[0]), len(y[0]))
        for other_key, particle in sorted(cls._registry.items(), cmp=compare):
            if key.startswith(other_key):
                suffix = key[len(other_key):]
                return cls(*(form + suffix for form in particle.forms))
        raise KeyError('There is no guessable particle')

    @property
    def after_vowel(self):
        return self.basic()

    @property
    def after_consonant(self):
        try:
            return self.forms[1]
        except IndexError:
            return self.basic()

    @property
    def after_rieul(self):
        try:
            return self.forms[2]
        except IndexError:
            return self.basic()

    def naive(self):
        rv = []
        unique_forms = list(set(self.forms))
        for forms in zip(unique_forms[:-1], unique_forms[1:]):
            length = map(len, forms)
            if len(set(length)) == 1:
                # such as "을(를)", "를(을)", "(을)를", "(를)을"
                rv.append(u'{0}({1})'.format(*forms))
                rv.append(u'{1}({0})'.format(*forms))
                rv.append(u'({0}){1}'.format(*forms))
                rv.append(u'({1}){0}'.format(*forms))
            else:
                # such as "(으)로"
                x = int(length[0] > length[1])
                args = forms[1 - x].rstrip(forms[x]), forms[x]
                rv.append(u'({0}){1}'.format(*args))
        return tuple(rv)

    def pick_allomorph_after_char(self, char):
        final = hangul.get_final(char)
        if not final:
            return self.after_vowel
        elif final == u'ㄹ':
            return self.after_rieul
        else:
            return self.after_consonant

    @define_allomorph_picker(suffix_of=Noun)
    @define_allomorph_picker(suffix_of=NumberWord)
    @define_allomorph_picker(suffix_of=Loanword)
    def pick_allomorph_after_substantive(self, substantive):
        return self.pick_allomorph_after_char(substantive.read()[-1])



# -*- coding: utf-8 -*-
"""
    korean.morphology.substantive
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyright: (c) 2012 by Heungsub Lee
    :license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import
import re

from .morpheme import Morpheme
from ..hangul import is_hangul


__all__ = ['Substantive', 'Noun', 'NumberWord', 'Loanword']


class Substantive(Morpheme):
    """A class for Korean substantive that is called "체언" in Korean."""

    def __format__(self, spec):
        """:class:`Substantive`'s custom formatter appends the correct particle
        after the substantive string using particle format spec such as
        ``{0:은}`` or ``{1:로}``:

            >>> format(Noun(u'엄마'), u'을')
            u'엄마를'
            >>> u'{0:은} {1:로}'.format(Noun(u'아들'), Noun(u'마을'))
            u'아들은 마을로'
            >>> u'{0:은} {1:로}'.format(Noun(u'아들'), Noun(u'산'))
            u'아들은 산으로'
        """
        from .particle import Particle
        from . import merge
        separated_spec = spec.split(':')
        if separated_spec[0] and is_hangul(separated_spec[0][0]):
            text = merge(self, Particle(separated_spec.pop(0)))
        else:
            text = unicode(self)
        try:
            spec = separated_spec[0]
        except IndexError:
            spec = ''
        return format(text, spec)


class Noun(Substantive):
    """A class for Korean noun that is called "명사" in Korean."""

    READING_PATTERN = re.compile(r'(?P<other>[^0-9]+)?(?P<number>[0-9]+)?')

    def read(self):
        """Reads a noun as Korean. The result will be Hangul.

            >>> Noun(u'레벨42').read()
            u'레벨사십이'
        """
        rv = []
        for match in self.READING_PATTERN.finditer(unicode(self)):
            if match.group('other'):
                rv.append(match.group('other'))
            if match.group('number'):
                rv.append(NumberWord(int(match.group('number'))).read())
        return ''.join(rv)


class NumberWord(Substantive):
    """A class for Korean number word that is called "수사" in Korean."""

    __numbers__ = {}
    __digits__ = {}

    def __init__(self, number):
        self.number = number

    def read(self):
        """Reads number as Korean.

            >>> NumberWord(1234567890).read()
            u'십이억삼천사백오십육만칠천팔백구십'
            >>> NumberWord.read_phases(0)
            u'영'
        """
        return ''.join(type(self).read_phases(self.number))

    @classmethod
    def read_phases(cls, number):
        """Reads number as Korean but seperates the result at each 10k.

            >>> NumberWord.read_phases(1234567890)
            (u'십이억', u'삼천사백오십육만', u'칠천팔백구십')
            >>> NumberWord.read_phases(0)
            (u'영',)
        """
        rv, phase = [], []
        digit = 0
        while True:
            single = number % 10
            if digit >= 4:
                try:
                    phase.append(cls.__digits__[digit])
                except KeyError:
                    pass
            if single:
                try:
                    phase.append(cls.__digits__[digit % 4])
                except KeyError:
                    pass
            number /= 10
            if not single and not number or \
               single == 1 and not digit or single > 1:
                phase.append(cls.__numbers__[single])
            if not number or digit % 4 == 3:
                if digit < 4 or len(phase) > 1:
                    rv.append(''.join(phase[::-1]))
                else:
                    rv.append(u'')
                phase = []
                if not number:
                    break
            digit += 1
        return tuple(rv[::-1])

    def basic(self):
        return unicode(self.number)

    def __format__(self, spec):
        try:
            return super(NumberWord, self).__format__(spec)
        except ValueError:
            return format(self.number, spec)


class Loanword(Substantive):
    """A class for loanword that is called "외래어" in Korean. This depends
    on `Hangulize <http://packages.python.org/hangulize>`_ which automatically
    transcribes a non-Korean word into Hangul.
    """

    def _import_hangulize(self):
        try:
            return self._hangulize
        except AttributeError:
            pass
        try:
            import hangulize
        except ImportError:
            raise ImportError('%s needs hangulize>=0.0.5' % type(self).__name__)
        self._hangulize = hangulize
        return hangulize

    def __init__(self, word, code=None, iso639=None, lang=None):
        hangulize = self._import_hangulize()
        self.lang = lang or hangulize.get_lang(code, iso639)
        super(Loanword, self).__init__(word)

    def read(self):
        """Transcribes into Hangul using `Hangulize
        <http://packages.python.org/hangulize>`_.

        >>> Loanword(u'Guido van Rossum', 'nld').read()
        u'히도 판로쉼'
        >>> Loanword(u'საქართველო', 'kat').read()
        u'사카르트벨로'
        >>> Loanword(u'Leonardo da Vinci', 'ita').read()
        u'레오나르도 다 빈치'
        """
        hangulize = self._import_hangulize()
        return hangulize.hangulize(self.basic(), lang=self.lang)



# -*- coding: utf-8 -*-
"""
    korean.morphology
    ~~~~~~~~~~~~~~~~~

    :copyright: (c) 2012 by Heungsub Lee
    :license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import
import sys
import types

from .. import hangul


__all__ = ['Morphology', 'Morpheme', 'Particle', 'Substantive', 'Noun',
           'NumberWord', 'Loanword', 'pick_allomorph', 'merge',
           'define_allomorph_picker']


class Morphology(object):

    _registry = {}

    @classmethod
    def _register_morpheme(cls, morpheme_cls):
        for attr in dir(morpheme_cls):
            if not attr.startswith('$'):
                continue
            for keyword, func in getattr(morpheme_cls, attr):
                keyword = (morpheme_cls,) + keyword
                if keyword in cls._registry:
                    raise ValueError('Already defined rule')
                try:
                    cls._registry[attr][keyword] = func
                except KeyError:
                    cls._registry[attr] = {keyword: func}

    @classmethod
    def _make_decorator(cls, tmp_attr, keyword):
        assert tmp_attr.startswith('$')
        frm = sys._getframe(2)
        def decorator(func):
            rule = (keyword, func)
            try:
                frm.f_locals[tmp_attr].append(rule)
            except KeyError:
                frm.f_locals[tmp_attr] = [rule]
            return func
        return decorator

    @classmethod
    def define_allomorph_picker(cls, prefix_of=None, suffix_of=None):
        if not (prefix_of or suffix_of):
            raise TypeError('prefix_of or suffix_of should be defined')
        elif bool(prefix_of) == bool(suffix_of):
            raise TypeError('Cannot specify prefix_of and suffix_of both')
        keyword = (prefix_of, suffix_of)
        return cls._make_decorator('$allomorph_pickers', keyword)

    @classmethod
    def pick_allomorph(cls, morpheme, prefix_of=None, suffix_of=None):
        prefix_type = prefix_of and type(prefix_of)
        suffix_type = suffix_of and type(suffix_of)
        keyword = (type(morpheme), prefix_type, suffix_type)
        func = cls._registry['$allomorph_pickers'][keyword]
        bound_func = types.MethodType(func, morpheme)
        return bound_func(prefix_of or suffix_of)

    @classmethod
    def merge(cls, prefix, suffix):
        try:
            prefix = cls.pick_allomorph(prefix, prefix_of=suffix)
        except KeyError:
            pass
        try:
            suffix = cls.pick_allomorph(suffix, suffix_of=prefix)
        except KeyError:
            pass
        if hangul.is_final(suffix[0]):
            prefix = prefix.read()
            splitted = hangul.split_char(prefix[-1])
            assert not splitted[2]
            mid = hangul.join_char((splitted[0], splitted[1], suffix[0]))
            return u'{0}{1}{2}'.format(prefix[:-1], mid, suffix[1:])
        else:
            return u'{0}{1}'.format(prefix, suffix)


pick_allomorph = Morphology.pick_allomorph
define_allomorph_picker = Morphology.define_allomorph_picker
merge = Morphology.merge


#: Imports submodules on the end. Because they might need :class:`Morphology`.
from .morpheme import Morpheme
from .particle import Particle
from .substantive import Substantive, Noun, NumberWord, Loanword



# -*- coding: utf-8 -*-
"""
Korean -- A library for Korean morphology
=========================================

Sometimes you should localize your project to Korean. But common i18n solutions
such as gettext are not working with non Indo-European language well. Korean
also has many morphological difference. "korean" a Python module provides
useful Korean morphological functions.

Do not use "을(를)" anymore
```````````````````````````

::

    >>> from korean import Noun
    >>> fmt = u'{subj:은} {obj:을} 먹었다.'
    >>> print fmt.format(subj=Noun(u'나'), obj=Noun(u'밥'))
    나는 밥을 먹었다.
    >>> print fmt.format(subj=Noun(u'학생'), obj=Noun(u'돈까스'))
    학생은 돈까스를 먹었다.

Links
`````

* `GitHub repository <http://github.com/sublee/korean>`_
* `development version
  <http://github.com/sublee/korean/zipball/master#egg=korean-dev>`_

"""
from setuptools import setup

import korean


setup(
    name=korean.__name__,
    version=korean.__version__,
    license=korean.__license__,
    author=korean.__author__,
    author_email=korean.__author_email__,
    url=korean.__url__,
    description='A library for Korean morphology',
    long_description=__doc__,
    platforms='any',
    packages=['korean', 'korean.morphology'],
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: Korean',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Localization',
        'Topic :: Text Processing :: Linguistic',
    ],
    test_suite='koreantests.suite',
    test_loader='attest:auto_reporter.test_loader',
    tests_require=['Attest', 'Babel'],
)



# -*- coding: utf-8 -*-
"""
    korean.l10n
    ~~~~~~~~~~~

    Helpers for localization to Korean.

    :copyright: (c) 2012 by Heungsub Lee
    :license: BSD, see LICENSE for more details.
"""
from __future__ import absolute_import
from functools import partial
from itertools import chain, product
import re
from StringIO import StringIO

from .morphology import Noun, NumberWord, Particle, pick_allomorph


__all__ = ['Proofreading', 'proofread', 'Template', 'patch_gettext']


class Proofreading(object):
    """A function-like class. These :meth:`__call__` replaces naive particles
    to be correct. First, it finds naive particles such as "을(를)" or
    "(으)로". Then it checks the forward character of the particle and replace
    with a correct particle.

    :param token_types: specific types to make as token.
    """

    def __init__(self, token_types):
        # TODO: support various token types
        pass

    def parse(self, text):
        """Tokenizes the given text with unicode text or :class:`Particle`.

        :param text: the string that has been written with naive particles.
        """
        tokens = []
        naive_particles = []
        particle_map = {}
        for particle in set(Particle._registry.itervalues()):
            for naive in particle.naive():
                particle_map[naive] = particle
                naive_particles.append(naive)
        particle_pattern = '(%s)' % '|'.join(map(re.escape, naive_particles))
        particle_pattern = re.compile(particle_pattern)
        prev_span = [0, 0]
        for match in particle_pattern.finditer(text):
            span = match.span()
            tokens.append(text[prev_span[1]:span[0]])
            tokens.append(particle_map[match.group(1)])
            prev_span = span
        try:
            tokens.append(text[span[1]:])
        except UnboundLocalError:
            tokens.append(text)
        return tuple(tokens)

    def __call__(self, text):
        """Do proofread. More information in :class:`Proofreading`.

        :param text: the string that has been written with naive particles.
        """
        stream = StringIO()
        for token in self.parse(text):
            if isinstance(token, Particle):
                stream.seek(-1, 2)
                noun = Noun(stream.read())
                try:
                    token = pick_allomorph(token, suffix_of=noun)
                except:
                    token = token.naive()[0]
            stream.write(token)
        return stream.getvalue()


#: Default :class:`Proofreading` object. It tokenizes ``unicode`` and
#: :class:`korean.Particle`. Use it like a function.
proofread = Proofreading([unicode, Particle])


class Template(unicode):
    """The :class:`Template` object extends :class:`unicode` and overrides
    :meth:`format` method. This can format particle format spec without
    evincive :class:`Noun` or :class:`NumberWord` arguments.

    Basically this example:

        >>> import korean
        >>> korean.l10n.Template(u'{0:을} 좋아합니다.').format(u'향수')
        u'향수를 좋아합니다.'

    Is equivalent to the following:

        >>> import korean
        >>> u'{0:을 좋아합니다.}'.format(korean.Noun(u'향수'))
        u'향수를 좋아합니다.'
    """

    def format(self, *args, **kwargs):
        args = list(args)
        for seq, (key, val) in chain(product([args], enumerate(args)),
                                     product([kwargs], kwargs.items())):
            if isinstance(val, unicode):
                seq[key] = Noun(val)
            elif isinstance(val, int):
                seq[key] = NumberWord(val)
        return super(Template, self).format(*args, **kwargs)

    def __repr__(self):
        return '<%s %s>' % \
               (type(self).__name__, super(Template, self).__repr__())


def patch_gettext(translations):
    """Patches Gettext translations object to wrap the result with
    :class:`korean.l10n.Template`. Then the result can work with a particle
    format spec.

    For example, here's a Gettext catalog for ko_KR:

    .. sourcecode:: pot

        msgid "{0} appears."
        msgstr "{0:이} 나타났다."

        msgid "John"
        msgstr "존"

        msgid "Christina"
        msgstr "크리스티나"

    You can use a particle format spec in Gettext messages after translations
    object is patched:

    .. sourcecode:: pycon

        >>> translations = patch_gettext(translations)
        >>> _ = translations.ugettext
        >>> _(u'{0} appears.').format(_(u'John'))
        u'존이 나타났다.'
        >>> _(u'{0} appears.').format(_(u'Christina'))
        u'크리스티나가 나타났다.'

    :param translations: the Gettext translations object to be patched that
                         would refer the catalog for ko_KR.
    """
    for meth in ['ugettext', 'ungettext']:
        def patched(orig, *args, **kwargs):
            return Template(orig(*args, **kwargs))
        patched.__name__ = meth
        orig = getattr(translations, meth)
        setattr(translations, meth, partial(patched, orig))
    return translations
